==> accountbook-3.0b1.tar.gz: accountbook-3.0b1/public/credit.rb <==
#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Account Book
# Copyright (c) 2002-2007 HORIKAWA Hisashi. All rights reserved.
#   mailto:vzw00011@nifty.ne.jp
#   http://www.nslabs.jp/

# 債務の消し込み

require "pcview.rb"
require APP_ROOT + '/models/invoice'
require APP_ROOT + '/models/payment_amount'
require APP_ROOT + '/models/account_title'
require LIBRARY_DIR + '/datesup'

class FirstData < Wafu::ActionForm
  def setup
    date = Date.today
    @mon = date.mon
    @day = date.day
    @cash_ac = nil

    j_ary = Invoice.find_by_sql([<<EOF, @app.user.id])
SELECT inv.id, inv.user_id, inv.partner_id, inv.date, inv.amount_total,
       sum(amo.amount) as paid
  FROM invoices inv LEFT JOIN payment_amounts amo 
                         ON inv.id = amo.invoice_id
  WHERE inv.user_id = ?
  GROUP BY inv.id, inv.user_id, inv.partner_id, inv.date, inv.amount_total
EOF
    # HAVING sum(amo.amount) <> inv.amount_total   amo.amountがナルのときに結果が空。

    @data = []
    j_ary.each {|r|
      next if r.amount_total == r.paid.to_i
      @data << {
        :invoice => r,
        :paid => r.paid.to_i
      }
    }
    @lines = @data.size
  end
  attr_reader :mon, :day, :cash_ac, :lines, :data
end

# 支払済みでない請求書一覧
class InvoiceList < Wafu::Action
  def execute(af)
    begin_day, end_day = UserSetting.get_addable_period(@app.user)
    model = {
      :period => "#{begin_day} 〜 #{end_day}",
      :sel_date => get_sel_date("", nil, af.mon, af.day),
      :sel_cash_ac => get_sel_active_items(@app.user, "cash", af.cash_ac.to_i) {|item| 
                                             item.cf_type == 4},
      :list => [],
      :notice => @app.get_flash("notice"),
    }
    if af.error?
      model[:error] = {:message => af.errors}
    end

    model[:lines] = af.lines
    (0..af.lines - 1).each {|i|
      r = af.data[i][:invoice]
      line_hash = {
        :date => r.date, # 購入日
        :partner => r.partner.name,
        :invname => "inv#{i}",
        :invid => r.id,
        :desc => r.journal_entries.first.comment,
        :amount => decimal_format(r.amount_total),
        :paid => decimal_format(af.data[i][:paid].to_i),
        :remain => decimal_format(r.amount_total - af.data[i][:paid].to_i),
        :payname => "pay#{i}",
        :payv => af.data[i][:pay]
      }
      model[:list] << line_hash
    }

    @app.page_out("payment_new.html", model)
  end
end

class PaymentAF < Wafu::ActionForm
  def validate
    @mon = @app.cgi.get('mon')
    @day = @app.cgi.get('day')
    @cash_ac = @app.cgi.get('cash')
    @lines = @app.cgi.get('lines').to_i
    raise "internal error" if @lines == 0

    err = []

    begin_day, end_day = UserSetting.get_addable_period(@app.user)
    d = get_full_day(@mon.to_i, @day.to_i, begin_day)
    err << "日付が不正です。" if !d
    err << "登録可能期間より前の日付は登録できません。" if d < begin_day
    err << "#{d}は登録可能期間より新しい日付です。メニューより締め処理をおこなってください。" if d > end_day

    # 相手ごとにマイナスの支払いは禁止。（請求書単位でのマイナスは許可）
    @data = []
    amount = {}
    count = 0
    (0..lines - 1).each {|i|
      inv = Invoice.find(@app.cgi.get("inv#{i}"))
      payv = @app.cgi.get("pay#{i}")
      @data << {:invoice => inv, :paid => inv.get_paid, :pay => payv}
      if payv.to_i != 0
        amount[inv.partner.id] = (amount[inv.partner.id] || 0) + payv.to_i
        count += 1
      end
    }
    err << "1件も支払いがありません。" if count == 0
    amount.each {|partner_id, amount|
      err << "#{Partner.find(partner_id).name}に対する支払いがマイナスです。" if amount < 0
    }

    return set_error(err)
  end
  attr_reader :mon, :day, :cash_ac, :lines, :data
end

class PaymentAction < Wafu::Action
  def execute(af)
    begin_day, end_day = UserSetting.get_addable_period(@app.user)
    d = get_full_day(af.mon.to_i, af.day.to_i, begin_day)
    raise if !d || d < begin_day || d > end_day
    user_setting = @app.user.user_setting

    Payment.transaction {
      payment = Payment.new(:user => @app.user, :date => d)
      payment.save!

      entry_no = (user_setting.entry_no_counter += 1)
      user_setting.save!
      cash_ac = AccountTitle.find(af.cash_ac)
      af.data.each {|line|
        if line[:pay].to_i != 0
          inv = line[:invoice]
          raise if inv.user.id != @app.user.id
          pa = PaymentAmount.new(:payment => payment, 
                                 :invoice => inv, 
                                 :amount => line[:pay].to_i)
          pa.save!
          payment.payment_amounts << pa

          e = JournalEntry.new(:user => @app.user,
                               :date => d,
                               :entry_no => entry_no,
                               :dr_code => user_setting.payable_item,
                               :cr_code => cash_ac.id,
                               :amount => pa.amount,
                               :remarks => inv.partner.name,
                               :comment => "支払い " + inv.journal_entries.first.comment,
                               :payment_amount => pa)
          e.save!
        end
      }
      raise if payment.payment_amounts.size == 0 # 1件もない
    }

    # @app.set_flash("notice", "支払いを登録しました。")
    @app.page_out("payment_create.html", {})
  end
end

class CreditApp < PCView
  def main()
    actions = {
      "*" => [FirstData, InvoiceList, nil],
      "pay" => [PaymentAF, PaymentAction, InvoiceList]
    }
    dispatch(actions)
  end
end

CreditApp.new.run()

==> accountbook-3.0.0.tar.gz: accountbook-3.0.0/public/credit.rb <==
#!/home/netsphere/bin/ruby
# -*- coding:utf-8 -*-

# Q's Web Account Book
# Copyright (c) 2002-2007,2010 HORIKAWA Hisashi. All rights reserved.
#   http://www.nslabs.jp/

# 債務の消し込み


require 'payment-common.rb'

class FirstData < Wafu::ActionForm
  def setup
    date = Date.today
    @mon = date.mon
    @day = date.day
    @cash_ac = nil

    unpaid_invoices = 
        Invoice.find :all,
            :select => "invoices.*, amo.paid",
            :joins => "LEFT JOIN (SELECT invoice_id, SUM(amount) AS paid FROM payment_amounts GROUP BY invoice_id) amo ON invoices.id = amo.invoice_id",
            :conditions => ["invoices.user_id = ? AND ((amo.paid IS NULL) OR invoices.amount_total <> amo.paid)", app.user.id]

    @data = []
    unpaid_invoices.each {|r|
      @data << {
        :invoice => r,
        :paid => r.paid.to_i
      }
    }
    @lines = @data.size
  end
  attr_reader :mon, :day, :cash_ac, :lines, :data
  attr_reader :payment
end


# レコードを生成
class PaymentCreate < Wafu::Action
  def execute(af)
    user_setting = app.user.user_setting
    begin_day, end_day = user_setting.get_addable_period
    d = get_full_day(af.mon.to_i, af.day.to_i, begin_day)
    raise if !d || d < begin_day || d > end_day

    Payment.transaction {
      payment = Payment.new(:user => @app.user, :date => d)
      payment.save!

      entry_no = (user_setting.entry_no_counter += 1)
      user_setting.save!
      cash_ac = AccountTitle.find(af.cash_ac)
      af.data.each {|line|
        if line[:pay].to_i != 0
          inv = line[:invoice]
          raise if inv.user.id != @app.user.id
          pa = PaymentAmount.new(:payment => payment, 
                                 :invoice => inv, 
                                 :amount => line[:pay].to_i)
          pa.save!
          payment.payment_amounts << pa

          e = JournalEntry.new(:user => @app.user,
                               :date => d,
                               :entry_no => entry_no,
                               :dr_code => user_setting.payable_item,
                               :cr_code => cash_ac.id,
                               :amount => pa.amount,
                               :remarks => inv.partner.name,
                               :comment => "支払い " + inv.journal_entries.first.comment,
                               :payment_amount => pa)
          e.save!
        end
      }
      raise if payment.payment_amounts.size == 0 # 1件もない
    }

    app.set_flash("notice", "支払いを登録しました。")
    app.redirect_to "credit.rb"
  end
end


PCView.new('actions' => {
  "*" => [FirstData, InvoiceList, nil],
  "pay" => [PaymentAF, PaymentCreate, InvoiceList]
}).run()
